/*
** graphics library code
** 6/22/09 changed SPI to prescal 1:1 --speed up
6/22/09 added Vmap array and clearDisplay function
9/26/09 adding bitmap processing
*/

#include <p24fj64ga002.h>
#include "graphics.h"
#include "Delay.h"

#include <stdlib.h>
#include "font.h"
void flipfont(const char *pf);

int cx, cy;
	unsigned char tempfont=0;
int YIN[128];
unsigned char ramfont[8];


#define _FAR __attribute__(( far))

// I/O definitions

#define CS   _LATB2    // CS output to EADOG128
#define RESET  _LATB3   // RESET output EADOG128
#define A0   _LATB5   // Ao output to EADOG128
const char init[] = {0x40,    //Display start line 0
                     0xa1,    //ADC reverse
                     0xc0,    //Normal COM0...COM63
                     0xa6,    //Display normal
                     0xa2,    //Set Bias 1/9 (Duty 1/65)
                     0x2f,    //Booster, Regulator and Follower On
                     0xf8,    //Set internal Booster to 4x
                     0x00,
                     0x27,    //Contrast set
                     0x81,
                    // 0x0f,  good BW
                     0x18,  //good amber
                   //0x12,  //good for green yellow
                     0xac,    //No indicator
                     0x01,
                     0xaf,    //Display on
                     0xb0,    //Page 0 einstellen
                     0x10,    //High-Nible der Spaltenadresse
                     0x00     //Low-Nible der Spaltenadresse
                     };
#define X0 0
#define Y0 (VRES/2)
#define PI 3.141592654f


int xus;
int yus;
 float xf;
float  yf;
unsigned char _FAR Vmap[HRES][VRES/8];  // array 128 bytes by 8 bytes or 1024 bytes

void initSD( void)
{
   int i=0;
 //  	 AD1PCFG = 0xffff;   // initialize to all digital
   	CS =1;
   _TRISB2 =0;
   A0=0;
   	_TRISB5 =0;
   RESET =1;
    _TRISB3 = 0;        // make Card select an output pin
    Delayms(10);
    RESET =0;
    Delayms(10);
    RESET =1;
   
    // init the spi module for a slow (safe) clock speed first

	SPI2CON1 = 0x013F;  // CKE=1; CKP=0, sample middle, prescale 1:1

    SPI2STAT = 0x8000;  // enable the peripheral
     	CS =0;
    //initialize and enable the EADOG128
    for (i =0; i<sizeof init; i++)
    {
	    writeSPI(init[i]);
	    delay_5us();
	    
	   } 

	clearScreen();

			
	//draw a broken byte line stream				
	   goto_xy(0,3);
	  	setAO();
	   for (i=0; i<64;i++)
	   writeSPI(0x55);
	   for (i=0; i<64;i++)
	   writeSPI(0xaa);
}   // initSD

void clearScreen (void){
	
	clearVmap();
		dumpVmap();	
	
	
	
}	
// send one byte of data and receive one back at the same time
unsigned char writeSPI( unsigned char b)
{
    SPI2BUF = b;					// write to buffer for TX
    while( !SPI2STATbits.SPIRBF);	// wait for transfer to complete
    return SPI2BUF;    				// read the received value
}// writeSPI

void clearDisplay(void) {
	
	 //clear all display ram
	   int y=0;
	   int i=0;
	    for (y =0;y<8;y++)
	 {
	   goto_xy(0,y);
	   setAO();		//data mode
	   	for (i=0; i<128;i++)
	   	writeSPI(0x00); 
		}							

}
void clearVmap( void) {
	int y=0;
	int x=0;
	    for (x =0;x<128;x++)
	    	for (y=0; y<8; y++)
	    	{
		    Vmap [x][y] =0;
				}
		    	
  }	
	
void dumpVmap( void)
{
	//dump Vmap onto display
	   int y=0;
	   int x=0;
	    for (y =0;y<8;y++)
	 {
	   goto_xy(0,y);
	   setAO();		//data mode
	   	for (x=0; x<128;x++)
	   	writeSPI(Vmap[x][y]); 
		}				
	
	
}
//plots pixel point into processor ram x =0 to 127 y =0 to 63
void Plot (unsigned char x, unsigned y) 
{
	if ((x<HRES)&& (y<VRES)) {	
	unsigned char temp = 0;
	temp = Vmap[x][y/8];  //retrive byte
	temp =temp| (0x01 << (y%8))	; //or appropiate bit
	Vmap[x] [y/8] = temp;			//write back
	}
}			
	
	
		

// stuff lifted from EADOG driver*************************************

void goto_xy(char x, char y)
{
  char set[] = {0xb0, 0x10, 0x00};    //Befehlsarray
  set[0] = set[0] + y;                //Y - Position auf Steuerbefehl addieren
  set[2] = set[2] + (x & 0x0f);       //Low - Nibble auf Steuerbefehl addieren
  set[1] = set[1] + (x >> 4);         //High - Nibble auf Steuerbefehl addieren
 
   	clearA0();			//cmd mode
	writeSPI(set[0]);	
	writeSPI(set[1]);
	writeSPI(set[2]);
}


//*************end of EADOG Driver**********************************

void enableCS(void) {
		CS =0;
	
}	

void disableCS(void) {
		CS =1;
	
}	
void setAO(void) {
		A0 =1;
	
}	

void clearA0(void) {
		A0=0;
	
}

//Bresenham algorithm --line drawing algorithm uses only integers
#define abs( a)     (((a)> 0) ? (a) : -(a))

void line( int x0, int y0, int x1, int y1)
{
     int steep, t ;
     int deltax, deltay, error;
     int x, y;
     int ystep;

     steep = ( abs(y1 - y0) > abs(x1 - x0));
     
     if ( steep )
     { // swap x and y
         t = x0; x0 = y0; y0 = t;
         t = x1; x1 = y1; y1 = t;
     }
     if (x0 > x1) 
     {  // swap ends
         t = x0; x0 = x1; x1 = t;
         t = y0; y0 = y1; y1 = t;
     }
     
     deltax = x1 - x0;
     deltay = abs(y1 - y0);
     error = 0;
     y = y0;
     
     if (y0 < y1) ystep = 1; else ystep = -1;
     for (x = x0; x < x1; x++)
     {
         if ( steep) Plot(y,x); else Plot(x,y);
         error += deltay;
         if ( (error<<1) >= deltax)
         {
             y += ystep;
             error -= deltax;
         } // if 
     } // for
} // line 



// this function plots x*sinx
void plotxsinx(void) {
	int x=0;
for( x=0; x<128; x++) //plots x*sin(x)
    {
        xf= (2 * PI / 50) * (float) x;
         yf=  50.0 / ( 8 * PI) * xf * sin( xf); 
        yus =(int) yf+Y0;
     Plot (x,yus);		//function
    
  }	
} 

void random_line_demo( void) {
 int i,x;
      srand( 12);
    for ( x=0; x<1; x++) {
         clearScreen();
       
         
         
        line( 0, 0, 0, VRES-1);
        line( 0, VRES-1, HRES-1, VRES-1);
        line( HRES-1, VRES-1, HRES-1, 0);
        line( 0, 0, HRES-1, 0);
        for( i = 0; i<50; i++) 
        {	
            line( rand()%HRES, rand()%VRES, rand()%HRES, rand()%VRES);
            
    		dumpVmap();
    			delay() ; //100s msec
								}
		}
			delay() ; //100s msec
				delay() ; //100s msec
					delay() ; //100s msec
					
}
void Rectangle ( int left, int top, int right, int bottom) {

	//draw rectangle one line at a time
	line( left,top, right,top);    //top line
	line ( left,bottom,right,bottom); //bottom line
	line (left,top, left,bottom);  // left side line
	line ( right,top, right, bottom); //right side line

	
	
}
void Bar ( int left, int top, int right, int bottom) {

	//draw rectangle one line at a time
	line( left,top, right,top);    //top line
	line ( left,bottom,right,bottom); //bottom line
	line (left,top, left,bottom);  // left side line
	line ( right,top, right, bottom); //right side line
		int y=0;
		int x=0;
	    for (x =0;x<128;x++)    //fill 
	    	for (y=0; y<64; y++)
	    	{
		    if ((( x>=left) && (x<=right)) && ((y <= bottom) && (y >=top)))
		    Plot(x,y);
				}
		    	
  }	
	




void Circle ( int xcenter, int ycenter, int radius) { 

	 int x, y, r2;
 	r2 = radius * radius;
	for (x = -radius; x <= radius; x++) {
            y = (int) (sqrt(r2 - x*x) + 0.5);
            Plot( xcenter + x, ycenter + y);
            Plot( xcenter + x, ycenter - y);
}
}	
void rightTriangle ( int topx, int topy, int rightx, int righty) {

	//draw rectangle one line at a time
	line( topx,topy, rightx,righty );    //draw hypotenuse
	line ( topx,righty,topx,topy); //draw perpendicular
	line (topx,righty, rightx,righty);  // draw base
	
}

void Triangle ( int topx, int topy, int rightx, int righty) {
	int base =0;
	base = 2* rightx-topx;
	//draw rectangle one line at a time
	line( topx,topy, rightx,righty );    //draw hypotenuse
	//line ( topx,righty,topx,topy); //draw perpendicular
	line (topx-base/2,righty, rightx,righty);  // draw base
	line (topx-base/2, righty, topx,topy);// draw hypotenuse
	
}


//bit map handing rotuine for 32x32 pixels

void convertrow ( unsigned long tlong, unsigned char row,unsigned char  col ) 
{
	int m=0;
	unsigned char bitpos;
	unsigned char temp;
	if ( row < 8 ) {
	if (row ==0) bitpos =0x01;
	if (row ==1) bitpos =0x02;
	if (row ==2) bitpos =0x04;
	if (row ==3) bitpos =0x08;
	if (row ==4) bitpos =0x10;
	if (row ==5) bitpos =0x20;
	if (row ==6) bitpos =0x40;
	if (row ==7) bitpos =0x80;
	}
	else {
		
		temp = (row%8);
	 
	if (temp ==0) bitpos =0x01;
	if (temp==1) bitpos =0x02;
	if (temp ==2) bitpos =0x04;
	if (temp ==3) bitpos =0x08;
	if (temp ==4) bitpos =0x10;
	if (temp ==5) bitpos =0x20;
	if (temp ==6) bitpos =0x40;
	if (temp ==7) bitpos =0x80;
			
		
	}	

	
	unsigned long Lrotate = 0x000001;

	unsigned char tempvalue =0;
	for (m =0; m <32; m++) {
		tempvalue = Vmap[col+m][row/8]; 
	if ( (Lrotate & tlong) >=1)	{
	Vmap[col+m][row/8] = tempvalue | bitpos;	
			
	}		
	Lrotate =Lrotate <<1;

		
	}	

}	
void drawbitmap32x32 ( unsigned long *slp,unsigned char xoff, unsigned char yoff) {
	
	int m =0;
	unsigned char xplace = xoff;
	unsigned char yplace = yoff;
	
	long unsigned tlong;

	for ( m=0; m<32; m++) {
			tlong = *slp;

				//do domething with bytes to convert proper postion in VMAP
				convertrow (tlong, yplace,  xplace);  // take 32 bit word and set appropiate bit postion in 32 columns of display
				xplace =xoff;
				yplace++;
				slp++;
		
	}
	
	
	
}	

//text handling routines
void putcV( int a)
{
      int i;
   const char *pf;
//    // 1. check if char in range
    a -= F_OFFS;
    if ( a < 0)         a = 0;
    if ( a >= F_SIZE)   a = F_SIZE-1; 
     
    //zero ramfont
    
    for (i=0; i<7; i++) {
	  ramfont[i] =0 ;  
	   } 
    //int i, *p;  cx and cy are byte postions
  
    // 2. check page boundaries 
    if ( cx >= HRES/8)          // wrap around x
    {
        cx = 0;
        cy++;   
    } 
    if ( cy >= VRES/8)          // wrap around y (or scroll?)
        cy = 0;
        
  
   a= a<<3;
    pf = &Font8x8[a];
    flipfont( pf);
//      pf = &Font8x8[a];
    // 4. copy one by one each line of the character on the screen
    unsigned char temp;
    for ( i=0; i<8; i++)
    {

        temp = Vmap[cx*8+i][cy];
       // temp |= *pf++;
       temp =ramfont[7-i];  //found that foont is backwards--must adjust
        Vmap[cx*8+i][cy] =temp;
  
    } // for
    
    // increment cursor position
    cx++;
} // putcV

void putsV( unsigned char *s)
{
    while (*s)
        putcV( *s++);
} // putsV
//this is my attempt to flip rom font to ram font-still resutls in bit pattern backwards 
void flipfont(const char *pf)
{    

	unsigned char  rotate =0;
	unsigned char test =0;
	unsigned char fontbit =0;
	int i =0;
	int x =0;
	fontbit =0x01;
	for (i=0; i<8;i++)
	{
	tempfont = *pf++;
	rotate =0x01;
	
  	for (x=0; x<8; x++) 
  	{
	  test = tempfont & rotate;
	  if (test)
	  {
		  ramfont[x] =ramfont[x] |fontbit;
		 } 	
		
		 	rotate =rotate<<1;		 
	  	
	  }	
 fontbit =fontbit <<1;
 	
		
		
		
	}	
	






}
//example 	plotsignal(&temp, 64,64,-20);
void plotsignal(int *s, int number, int ox, int oy) {
 int x=0;
//int y=0;
 int z;
 
	//convert fractional to scale suitable for display
	for( x=0; x<number; x++) {
		z = *s;
		s++;
	
	YIN[x] =  ((z )/16); 

	 Plot (x+ox,YIN[x]+oy);
	}
}	