/*
**	kibacorp inc graphics library Test used for EA DOGM128-6 display
** modified 3/14 for new experimenter board with exception of RPX
6/21/09 thk Modifed to run EA DOG128-6  SPI port
6/22/09 thk speed up SPI port 1;1 prescale CKE and CKP changes not necessary
6/22/09 thk added VMap array for on chip memory map of display
7/2/09 thk begin to transition to graphics library -rectangle (x,y,x,y) circle (x,y,r)
7/3/09 thk fill bar , draws isocolles and right triangle with fill
7/19/09 thk  added font generation and completed demo
8/11/09 thk added yellow/green display and attempting to set constrast--works
8/21/08 thk added lcd.c and lcd.h to run dual displays
9/26/09 thk rewrite to accomodate graphicsd library articilte for Nuts and volts
9/26/09 thk adding bit map 32x32 display capability
9/27/09 added bitmaps demo
12/23/09 cleanup and added waveform viewer option
12/23/09 SNAPSHOT Viewer
*/

#include <p24fj64ga002.h>
// I/O and graphics drivers
#include <libq.h>  //to handle fractional

#include "graphics.h"
#include "Delay.h"
#include "lcd.h"
#include "adc.h"

// bit maps
#include "shuttle.h"
#include "saturn.h"
#include "sun.h"
#include "telescope.h"
#include "rocket.h"
#include "robot.h"
#include "voyager.h"
#include "earth.h"
#include "spacecraft.h"
#include "moon.h"
#include "spacestation.h"
#include "pioneer.h"
#include "jupiter.h"
#include "robbie.h"
#include "bitmap_key.h"
#include "mexican32x32.h"
#include "kibacorp.h"

// canned 16x3 character display strings used during demo
const char Message0[]=    " Graphic demo";
const char Message1[]=    " Random lines";
const char Message2[]=    " Waveform demo";
const char Message3[]=    " Shapes demo";
const char Message4[]=    " Fonts demo";
const char Message5[]=    " Thank You";
const char Message6[]=    " Bitmaps demo";
const char snap1[] = "SnapShot Data ";
const char snap2[] = "   Viewer ";
const char snap3[] = " ";
	int temp [128];  //used to store data for wavefromer viewer



main( void)
{
	
	
    // Experimenter Programm Expansion Bus I/O  initializations
    // uses PIC24F SPI #2 port
  	CLKDIVbits.RCDIV = 0;
  	AD1PCFG = 0xffff;   // initialize to all digital
    
__builtin_write_OSCCONL(OSCCON & 0xbf) ; //clear bit 6


	RPINR22bits.SDI2R =  9;  // make Pin RP9 SDI2



	RPOR3bits.RP7R =10;   //SDO2  RP7
	RPOR4bits.RP8R = 11; // SCK2OUT RP8

__builtin_write_OSCCONL(OSCCON | 0x40) ; //set bit 6

	LCD_Initialize();		//initialize 16x3 LCD
    initSD();				// initialize graphic LCD and SPI communications
    initADC(pin9);
   	clearScreen();			//clear graphics screen
   	int i=0;
  	clear_display(); 
	position_cursor(0);
	write_string_LCD(Message0);
// modified to just display data on screen when SW1 is depressed
	clear_display(); 
	position_cursor(0);
	write_string_LCD(snap1);
	position_cursor(16);
	write_string_LCD(snap2);
	position_cursor(32);
	write_string_LCD(snap3);

while (1) {
	clearScreen();
	
	for ( i=0; i<128;i++)
	{
	temp[i]=	readADC( pin9);
	}
	plotsignal(&temp, 128,0,0);
	dumpVmap();
	big_delay();
}	
	
	
	  
    
 while(1) { 
	 //1. output message0 string on char LCD
	clear_display(); 
	position_cursor(0);
	write_string_LCD(Message0);
	//1. output intro splash screen on graphics display****************
   	AT( 0, 0) ; 
   	putsV( " Graphics demo");
   	AT(0,1);
    putsV( "  for the  ");
    AT(0,2);  
   putsV( " 16 bit Micro"); 
   AT(0,3)
   putsV( " Experimenter"); 
   drawbitmap32x32 ( &bitmapshuttle,0,32);
   drawbitmap32x32 ( &bitmapsun,32,32);
   drawbitmap32x32 ( &bitmapmoon,64,32);
   drawbitmap32x32 ( &bitmaprocket,92,32);
    dumpVmap();
    // wait couple of seconds before clearing and moving on
    
    big_delay();
  
	//2. output message1 string on char LCD
	clear_display(); 
	write_string_LCD( Message1);
	//2. output random line demo on graphics display**************** 
 	AT( 0, 2) ;
 	 putsV( " Random Line" );	
  	AT( 0, 3) ;
  	putsV( " Demostration " );
  	dumpVmap();
  	// wait couple of seconds before clearing and moving on
  	big_delay();
   	clearScreen();
   	//2. start random line demo
    random_line_demo();  //does a complete series of random line displays
    //3. output message2 string on char LCD
	clear_display(); 
	position_cursor(0);
	write_string_LCD(Message2);
  	clearScreen();
 	plotxsinx();		// output calculated raised cosine wave onto display
    AT( 0, 7) ;
    //3. Output Waveform and ext to graphics screen
  	putsV( "Waveform Demo");
  	dumpVmap();
  	//4. start random line demo
 	big_delay();
 	clearScreen();
 	//5. added bitmaps demo *************
 	clear_display(); 
	write_string_LCD(Message6);
   AT(0,0)
   putsV( "32x32 bitmaps");
   //5. stack up 4 bitmaps in a row on display round 1
   drawbitmap32x32 ( &bitmapshuttle,0,20);
   drawbitmap32x32 ( &bitmapsun,32,20);
   drawbitmap32x32 ( &bitmapmoon,64,20);
   drawbitmap32x32 ( &bitmaprocket,92,20);
   dumpVmap();
    big_delay();
    big_delay();
 	clearScreen();
 	AT(0,0)
   putsV( "32x32 bitmaps");
   //5. stack up 4 bitmaps in a row on display round 2
   drawbitmap32x32 ( &bitmaptelescope,0,20);
   drawbitmap32x32 ( &bitmaprobot,32,20);
   drawbitmap32x32 ( &bitmaprobbie,64,20);
   drawbitmap32x32 ( &bitmapvoyager,92,20);
 	dumpVmap();
	 big_delay();
   big_delay();
 	clearScreen();
	 AT(0,0)
   putsV( "32x32 bitmaps");
   //5. stack up 4 bitmaps in a row on display round 3
   drawbitmap32x32 ( &bitmapspacestation,0,20);
   drawbitmap32x32 ( &bitmappioneer,32,20);
   drawbitmap32x32 ( &bitmapsaturn,64,20);
   drawbitmap32x32 ( & bitmapspacecraft,92,20);
   
 //*********bit maps demo end ****************
  dumpVmap();
 	big_delay();
   big_delay();
  //6.  start shapes demo
 	clearScreen();
	Triangle (80,5,100,50);
	 rightTriangle ( 12,4, 30, 20); //small right triangle
 	Rectangle (34,8,48,20);
 	Bar ( 110,40,120,60); //small rectangle below it
 	 Circle ( 110,22,15);
   //output message 3 on char LCD
	 clear_display(); 
	 position_cursor(0);
	write_string_LCD(Message3);
	AT( 0, 7) ;
  	putsV( " Shapes Demo");
    dumpVmap();
    big_delay();
 	clearScreen();
  //7. output message 4  on char LCD FONT demo
	 clear_display(); 
	 position_cursor(0);
	write_string_LCD(Message4);
	  AT( 0, 0) ;
      putsV( "   8x8 Font" );	
      AT( 0, 1) ;
      putsV( " Demostration " );
      AT (0,3);
     putsV("0123456789");
      AT (0,4);
      putsV("ABCDEFGHIJKLMNOPQRSTUVWXYZ");
      putsV("abcdefghijklmnopqrstuvwxyz");
      putsV("!@#$%^&*()+");
     dumpVmap();  
     big_delay();
       big_delay();
 clearScreen();
  //#8 "thank You" output message 5 on char LCD
	 clear_display(); 
	position_cursor(0);
	write_string_LCD(Message5);
  AT( 0, 2) ;
  putsV( " Thank You" );	
    dumpVmap();
   big_delay();
 } //go back and repeat
 

} // main

